@@ -17,7 +17,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-#  $Id: dpkg-cross.pl,v 1.26 2005/05/13 18:43:34 yoush-guest Exp $
+#  $Id: dpkg-cross.pl,v 1.39 2006/04/18 07:14:35 yoush-guest Exp $
 
 # Determine if the system wide or user defined cross-compile configuration
 # have to be read.
@@ -31,44 +31,45 @@ $conffile = (-e $private_conffile) ? $private_conffile : $conffile;
 # List of variables which can be overriden by the cross-compile
 # definitions.
 @intern_vars = qw( crossbase crossprefix crossdir crossbin crosslib crosslib64 crossinc
-		crossinfo maintainer default_arch removedeps keepdeps compilerpath );
+		maintainer default_arch removedeps keepdeps compilerpath );
 # Avoid warnings about unused @intern_vars variables.
 foreach my $var_ ( @intern_vars ) {
 	eval( "\$$var_ = ''" );
 }
 
 # Version of dpkg-cross environment.
-$DPKGCROSSVERSION = "1.18";
+$DPKGCROSSVERSION = "1.26";
 
 # Convertion table for Debian GNU/Linux architecture name (``$arch'') to GNU
 # type.
 %archtable = (
-	'i386' => 'i386-linux',
-	'sparc' => 'sparc-linux',
-	'sparc64' => 'sparc64-linux',
-	'alpha' => 'alpha-linux',
-	'm68k' => 'm68k-linux',
-	'arm' => 'arm-linux',
-	'powerpc' => 'powerpc-linux',
-	'ppc' => 'powerpc-linux',
-	'mips' => 'mips-linux',
-	'mipsel' => 'mipsel-linux',
-	'sh3' => 'sh3-linux',
-	'sh4' => 'sh4-linux',
-	'sh3eb' => 'sh3eb-linux',
-	'sh4eb' => 'sh4eb-linux',
-	'hppa' => 'hppa-linux',
+	'i386' => 'i486-linux-gnu',
+	'sparc' => 'sparc-linux-gnu',
+	'sparc64' => 'sparc-linux-gnu',
+	'alpha' => 'alpha-linux-gnu',
+	'm68k' => 'm68k-linux-gnu',
+	'arm' => 'arm-linux-gnu',
+	'armeb' => 'armeb-linux-gnu',
+	'powerpc' => 'powerpc-linux-gnu',
+	'ppc' => 'powerpc-linux-gnu',
+	'mips' => 'mips-linux-gnu',
+	'mipsel' => 'mipsel-linux-gnu',
+	'sh3' => 'sh3-linux-gnu',
+	'sh4' => 'sh4-linux-gnu',
+	'sh3eb' => 'sh3eb-linux-gnu',
+	'sh4eb' => 'sh4eb-linux-gnu',
+	'hppa' => 'hppa-linux-gnu',
 	'hurd-i386' => 'i386-gnu',
-	's390' => 's390-linux',
-	's390x' => 's390x-linux',
-	'ia64' => 'ia64-linux',
- 	'm32r' => 'm32r-linux',
+	's390' => 's390-linux-gnu',
+	's390x' => 's390-linux-gnu',
+	'ia64' => 'ia64-linux-gnu',
+ 	'm32r' => 'm32r-linux-gnu',
 	'openbsd-i386' => 'i386-openbsd',
 	'freebsd-i386' => 'i386-freebsd',
 	'darwin-powerpc' => 'powerpc-darwin',
 	'darwin-i386' => 'i386-darwin',
 	'win32-i386' => 'i386-cygwin',
-	'amd64' => 'x86_64-linux');
+	'amd64' => 'x86_64-linux-gnu');
 
 # Regexps to parse 'file' output to detect arch of ELF binary
 # Note that it is not always possibe to restore Debian architecture
@@ -84,6 +85,7 @@ $DPKGCROSSVERSION = "1.18";
 	'alpha' => 'ELF 64-bit LSB .* Alpha',
 	'm68k' => 'ELF 32-bit MSB .* 680[02]0',
 	'arm' => 'ELF 32-bit LSB .* ARM',
+	'armeb' => 'ELF 32-bit MSB .* ARM',
 	'powerpc' => 'ELF 32-bit MSB .* PowerPC',
 	'mips' => 'ELF 32-bit MSB .* MIPS',
 	'mipsel' => 'ELF 32-bit LSB .* MIPS',
@@ -92,32 +94,35 @@ $DPKGCROSSVERSION = "1.18";
 	's390x' => 'ELF 64-bit MSB .* S.390',
 	'ia64' => 'ELF 64-bit LSB .* IA-64',
  	'm32r' => 'ELF 32-bit MSB .* M32R',
-	'amd64' => ' ELF 64-bit LSB .* x86-64',
+	'amd64' => 'ELF 64-bit LSB .* x86-64',
+	'w32-i386' => '80386 COFF',
 	'AR' => 'current ar archive');
 
 # Lists of possible crossprefix'es for each architecture
 # FIXME: table is incomplete.
 %crossprefixtable = (
-	"i386" => [ "i386-linux-", "i486-linux-", "i386-gnu-" ],
-	"sparc" => [ "sparc-linux-", "sparc64-linux-" ],
-	"sparc64" => [ "sparc64-linux-", "sparc-linux-" ],
-	"alpha" => [ "alpha-linux-" ],
-	"m68k" => [ "m68k-linux-" ],
-	"arm" => [ "arm-linux-" ],
-	"powerpc" => [ "powerpc-linux-", "ppc-linux-" ],
-	"mips" => [ "mips-linux-", "mips64-linux-" ],
-	"mipsel" => [ "mipsel-linux-", "mips64el-linux-" ],
-	"hppa" => [ "hppa-linux-" ],
-	"s390" => [ "s390-linux-", "s390x-linux-" ],
-	"s390x" => [ "s390x-linux-", "s390-linux-" ],
-	"ia64" => [ "ia64-linux-" ],
-	"m32r" => [ "m32r-linux-" ],
-	"amd64" => [ "amd64-linux-", "x86-64-linux-" ]);
+	"i386" => [ "i486-linux-gnu-", "i386-linux-gnu-", "i486-linux-", "i386-linux-", "i486-gnu-", "i386-gnu-" ],
+	"sparc" => [ "sparc-linux-gnu-", "sparc-linux-", "sparc64-linux-gnu-", "sparc64-linux-" ],
+	"sparc64" => [ "sparc64-linux-gnu", "sparc64-linux-", "sparc-linux-gnu-", "sparc-linux-" ],
+	"alpha" => [ "alpha-linux-gnu-", "alpha-linux-" ],
+	"m68k" => [ "m68k-linux-gnu-", "m68k-linux-" ],
+	"arm" => [ "arm-linux-gnu-", "arm-linux-" ],
+	"armeb" => [ "armeb-linux-gnu-", "armeb-linux-" ],
+	"powerpc" => [ "powerpc-linux-gnu-", "powerpc-linux-", "ppc-linux-gnu-", "ppc-linux-" ],
+	"mips" => [ "mips-linux-gnu-", "mips-linux-", "mips64-linux-gnu-", "mips64-linux-" ],
+	"mipsel" => [ "mipsel-linux-gnu-", "mipsel-linux-", "mips64el-linux-gnu-", "mips64el-linux-" ],
+	"hppa" => [ "hppa-linux-gnu-", "hppa-linux-" ],
+	"s390" => [ "s390-linux-gnu-", "s390-linux-", "s390x-linux-gnu-", "s390x-linux-" ],
+	"s390x" => [ "s390x-linux-gnu-", "s390x-linux-", "s390-linux-gnu-", "s390-linux-" ],
+	"ia64" => [ "ia64-linux-gnu-", "ia64-linux-" ],
+	"m32r" => [ "m32r-linux-gnu-", "m32r-linux-" ],
+	"amd64" => [ "amd64-linux-gnu-", "amd64-linux-", "x86_64-linux-gnu-", "x86_64-linux-",
+			"x86-64-linux-gnu-", "x86-64-linux-" ],
+	"w32-i386" => [ "i586-mingw32msvc-" ]);
 
 # All environment variables for compiler and binutils with their corresponding
 # application name which will be modified for cross-compiling.
-# For binutils, only LD and AS make sense: for strip we have
-# a wrapper, ar and ranlib are archiecture-indepndent.
+# For strip we have a wrapper, so it isn't mentioned here.
 %std_tools = (
 	CC => "gcc",
 	GCC => "gcc",
@@ -125,7 +130,11 @@ $DPKGCROSSVERSION = "1.18";
 	CPP => "cpp",
 	IMAKECPP => "cpp",
 	AS => "as",
-	LD => "ld");
+	LD => "ld",
+	AR => "ar",
+	NM => "nm",
+	RANLIB => "ranlib",
+	RC => "windres");
 
 # Contains variable definitions from ``$conffile'' distinguished by their
 # scope ("makeflags" or "environment") of definition.
@@ -241,10 +250,9 @@ sub get_architecture {
 
 ################################################################################
 # setup():
-#     Set global variables ``$arch'', ``$arch_cpu'', ``$arch_os'',
-#     ``$crossbase'', ``$crossbin'', ``$crossdir'', ``$crossinc'',
-#     ``$crossinfo'', ``$crosslib'', ``$crosslib64'', ``$crossprefix'',
-#     ``$deb_host_arch'', ``$compilerpath'' and ``$deb_host_gnu_type'' to
+#     Set global variables ``$arch'', ``$crossbase'', ``$crossbin'',
+#     ``$crossdir'', ``$crossinc'', ``$crosslib'', ``$crosslib64'', ``$crossprefix'',
+#     ``$compilerpath'' and ``$deb_host_gnu_type'' to
 #     defaults and substitude them with variables from ``@intern_vars'' and
 #     ``$arch''.
 #
@@ -257,22 +265,15 @@ sub setup {
 	
 	# Set ``$arch'' to defaults if not already specified.
 	$arch = get_architecture();
-	
-	# Determine OS and CPU from ``$arch''.
-	$deb_host_arch = $arch || die "$progname: Architecture is not specified.\n";
-	$deb_host_gnu_type = $archtable{$arch};
-	
-	$arch_cpu = $deb_host_gnu_type;
-	$arch_cpu =~ s/-.*$//;
-	$arch_os = $deb_host_gnu_type;
-	$arch_os =~ s/^.*-//;
+	die "$progname: Architecture is not specified.\n" unless ($arch);
+	$deb_host_gnu_type = $archtable{$arch};		# FIXME: should use dpkg-architecture here
 	
 	# Finalize, no subst possible crossbase.
 	$crossbase ||= "/usr";
 	
 	# Set defaults for internal vars, if not set ...
-	$crossprefix   ||= $ENV{'CROSSPREFIX'} || "${arch_cpu}-${arch_os}-";
-	$crossdir      ||= "\$(CROSSBASE)/${arch_cpu}-${arch_os}";
+	$crossprefix   ||= $ENV{'CROSSPREFIX'} || "${deb_host_gnu_type}-";
+	$crossdir      ||= "\$(CROSSBASE)/${deb_host_gnu_type}";
 	$crossbin      ||= "\$(CROSSDIR)/bin";
 	
 	if (exists $allcrossroots{$arch}) {
@@ -284,7 +285,6 @@ sub setup {
 	else {
 		$crosslib  ||= "\$(CROSSDIR)/lib";
 		$crossinc  ||= "\$(CROSSDIR)/include";
-		$crossinfo ||= "\$(CROSSLIB)/dpkg-cross-info";
 	}
 
 	$crosslib64 ||= $crosslib . "64";
@@ -461,18 +461,12 @@ sub setup_cross_env {
 	$makeflags_{'ARCH'} = $arch;
 	$ENV{'DPKGCROSSARCH'} = $arch;
 	
-	# Also set new `dpkg-architecture' environment veriables.
-	chomp ($tmp_ = `dpkg-architecture -a$arch -qDEB_HOST_ARCH 2>/dev/null`);
-	$makeflags_{'DEB_HOST_ARCH'} = $tmp_;
-	
-	chomp ($tmp_ = `dpkg-architecture -a$arch -qDEB_HOST_GNU_CPU 2>/dev/null`);
-	$makeflags_{'DEB_HOST_GNU_CPU'} = $tmp_;
-	
-	chomp ($tmp_ = `dpkg-architecture -a$arch -qDEB_HOST_GNU_SYSTEM 2>/dev/null`);
-	$makeflags_{'DEB_HOST_GNU_SYSTEM'} = $tmp_;
-	
-	chomp ($tmp_ = `dpkg-architecture -a$arch -qDEB_HOST_GNU_TYPE 2>/dev/null`);
-	$makeflags_{'DEB_HOST_GNU_TYPE'} = $tmp_;
+	# Set `dpkg-architecture' environment veriables.
+	foreach $var_ qw(DEB_HOST_ARCH DEB_HOST_ARCH_OS DEB_HOST_ARCH_CPU
+        		DEB_HOST_GNU_CPU DEB_HOST_GNU_SYSTEM DEB_HOST_GNU_TYPE) {
+		chomp ($tmp_ = `dpkg-architecture -a$arch -q$var_ 2>/dev/null`);
+		$ENV{$var_} = $tmp_;
+	}
 	
 	# Prepend /usr/share/dpkg-cross/ to PATH to make use of the strip wrapper.
 	# Also append $crossbin, so that cross binaries can be found, but
@@ -492,7 +486,8 @@ sub setup_cross_env {
 	
 	# Set standard variables for compilers and binutils.
 	foreach $var_ ( keys %std_tools ) {
-		$makeflags_{$var_} = $crossprefix . $std_tools{$var_};
+		$tool_ = $crossprefix.$std_tools{$var_};
+		$makeflags_{$var_} = find_in_path($tool_) ne "" ? $tool_ : $std_tools{$var_};
 	}
 
 	# Allow to use $crossprefix-gcc -E as preprocessor if $crossprefix-cpp
